/*A*/ /* Head  */
/****h*  parameter.c *********
 *
 *	NAME
 *    parameter.c
 *
 *	FUNCTION
 *    all functions for parameter parsing
 *
 *	CREATION DATE
 *	  4-Mar-1997
 *
 *  HISTORY
 *    040397 disconnected this part from main.c
 *    19mar02 WR: beautified source code, corrected typos
 *    2jun02 WR: started with -H and -S
 *		21-27oct04 WR: -m (beta 10)
 *    5jun6: WR+LK: 6x4* (beta 15)
 *
 *  TODO
 *
 *  BUGS 
 *
 *  BEAST METHODS
 *
 *    parseargs()
 *    help()
 *    copyright()
 *
*******
*/

/* {{{ */
/* for folding recognition */
/* }}} */

/*E*/
/*-----------------------------------------*/
/*A*/ /*  External routines and structures */

#include "define.h"

/*E*/
/*-----------------------------------------*/
/*A*/ /*  Global variables */

extern struct OPTIONS options;   /* for some options */

/*
   pointers to dynamic allocated memory
*/

/*E*/
/*-----------------------------------------*/
/*A*/char *convertMHSS(char *ss, int num_jug) {
	
	/* ss: siteswap (unfiltered from command line)
		num_jug: number of jugglers
		*/
	
	int ss_len, i, h;
	char ss2[SS_LEN];
	static char mhss[4*SS_LEN];
	
	// check 4 invalid characters:
	if (strpbrk(ss,",.[]<>()ABCDEFGHIJKLMNOPQRSTUVWXYZ"))
		return NULL;

	strcpy (ss2, ss);
	// make as many periods as jugglers:
	for (i = 1; i < num_jug; i++) {
		strcat(ss2,ss);
	}
	ss_len = strlen(ss2);

  h = 0;  // hand and index count differently, because of possible whitespace in input string
	for (i=0; i<ss_len; i++) {
		char buf[SS_LEN];
		int orig_throw;
		int dest_hand;
		int is_self;
		double throw;
		double b;
		
		if (ss2[i] == ' ') 
			continue;  // do nothing with space
		
		// original throw:
		orig_throw = ss2[i]-(ss2[i]>'9'?'a'-10:'0');
		// throw divided through number of jugglers:
		throw = (double)orig_throw/(double)num_jug;

		is_self = 0;
		
		if (modf(throw,&b) == 0.0) { // self
			is_self = 1;
		} else {
			if (num_jug == 2) {
				// add "p" for passing in 2-juggler-mode
				sprintf(buf," %sp",fracPass(throw));
			} else { // more than 2 jugglers
				dest_hand = (h+orig_throw) % (num_jug*2);
				if (self (dest_hand%(num_jug*2), h%(num_jug*2)))
					is_self = 1;
				else {
					char dhc;
					dhc = 'A' + (dest_hand %num_jug);
					// add char for destination juggler
					sprintf(buf," %s%c",fracPass(throw),dhc);
				}
			}
		}
		if (is_self) {
			if (throw > 9.)
				sprintf(buf," %c",((int)throw)+'a'-10);
			else
				sprintf(buf," %d",(int)throw);
		}
		strcat(mhss,buf);
    h++;
	} // for
	return mhss;
}
/*E*/
/*-----------------------------------------*/
/*A*/void copyright(){
/****** parameter.c/copyright *********
 *
 *	NAME
 *	  copyright
 *
 *	FUNCTION
 *    shows the copyright information
 *
 *  SYNOPSIS
 *    copyright()
 *
 *  RESULT
 *    text on console
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    help()
 *
 *******
 */
  printf(

         PROGNAME" is a converter for siteswap juggling programs\n\n"

         PROGNAME" "VERSION" compiled (%s - %s),\n"
         "Copyright (C) 1997 Martin Hoffmann\n"
				 "Some improvements 1997 - 2006 by Werner Riebesel\n\n"
   
         "This program is free software; you can redistribute it and/or modify\n"
         "it under the terms of the GNU General Public License as published by\n"
         "the Free Software Foundation; either version 2 of the License, or\n"
         "any later version.\n\n",__DATE__,__TIME__);
     
  printf(
         "This program is distributed in the hope that it will be useful,\n"
         "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
         "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
         "GNU General Public License for more details.\n\n");
     
  printf(
         "You should have received a copy of the GNU General Public License\n"
         "along with this program; if not, write to the Free Software\n"
         "Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");

} /* copyright() */
/*E*/
/*-----------------------------------------*/
/*A*/char *fracPass(double throw) {
	
	static char throwstring[20], buf[5];
	double integer, fraction;
	
	if (throw<10) {
		// return "9.5"
		sprintf(throwstring,"%.1f",throw);
	} else {
		// return "a.5"
		fraction = modf(throw, &integer);
		sprintf(throwstring,"%.1f", fraction);  // write "0.5"
		sprintf(buf,"%c",((int)integer)-10+'a');  
		throwstring[0] = buf[0]; // overwrite "0" with a letter
	}

	return throwstring;
	
}
/*E*/
/*-----------------------------------------*/
/*A*/void help(){

/****** parameter.c/help *********
 *
 *	NAME
 *	  help
 *
 *	FUNCTION
 *    shows a help text
 *
 *  SYNOPSIS
 *    help()
 *
 *  RESULT
 *    text on console
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    copyright()
 *
 *******
 */

  if (options.quiet) return;

  printf(
         PROGNAME" is a converter for siteswap juggling programs.\n"
         "Version "VERSION
         "\n\ncall: "PROGNAME" [-options] <site-swap> <outfile>\n\n"

         "Options :\n\n");

  printf(
         "-d    : shows some debug information (remember: this is still beta)\n"
//"-e<X> :*No endless loop. Add start and ending.\n"
         "-f    : program fails.\n"
         "-h<xx>: handpositions (c|1=casc  C|r|2=rev.casc.  b|3=backcross)\n"
         "-H<throwspec>: more detailled handpositions (see below)\n"
         "-i<xx>: input by file\n"
         "-l    : long handholding\n"
         );
  printf(
//"-m    :+Mark path with different coloured balls.\n"
				 "-m<x> : multi handed siteswap: use <x> hands (see below)\n" 
         "-n    : don't start jongl\n"
         "-o<x> : use this object\n"
         "-p<x> : number of people for passing (not necessary if all people pass)\n"
				 "        -p1 forces solo mode where 'p' is a throw and no passing postfix\n"
         "-r<f> : use <f> for radius between jugglers [in metres]\n"
         "-q    : quiet\n"
         "-s    : juggle synchronous.\n"
         "-S<throwspec> : handpositions for each throw of the siteswap pattern\n"
//"-w    :*smaller period\n"
         );
  printf(
         "\nthrowspec: <throw><spin><bounce><Z-rotation> with:\n"
         "  <throw> is one of: c,C,b (same as -h)\n"
         "  <spin> is one of: sp**, sp-1, sp00, sp+1, ... sp+N\n"
         "    sp**: spin depends on height (default)\n"
         "    sp-1: single reverse spin, sp+3: triple spin\n"
         "  <bounce> is one of: bo**, bo+1, ... bo+N, bo-1, ... bo-N\n"
         "    bo**: no bouncing, bo-1: force bounce, bo+2: double lift bounce\n"
         "  <Z-rotation> has the form Z<angle>, e.g. Z+45 or Z-07\n"
         "    Z-90: club points to the left, Z+00: straight, Z-90: to the right\n"
         "  All of the parameters are mandatory, and are to be repeated for\n"
         "  every hand (-H) or siteswap throw (-S). \n"
         "  Default is: csp**bo**Z+00 (13 characters per hand/throw)\n"
				 "\nstar notation is supported: 6x4* produces 6x4 46x\n"
				 "\nmulti handed siteswap: -m4 996 produces 8 club PPS\n"
         );
	printf(
//"+ means buggy,  * means not functional\n"
         "\nRemember: the left hand throws first. default-outfile: '"DEFOUTNAME"'\n\n");

	printf(
         PROGNAME" comes with ABSOLUTELY NO WARRANTY. This is free software,\n"
         "and you are welcome to redistribute it under certain conditions\n"
         "type `"PROGNAME" -c' for details.\n");
} /* help() */

/*E*/
/*-----------------------------------------*/
/*A*/int parseargs(int argc, char *argv[]){

/****** parameter.c/parseargs *********
 *
 *	NAME
 *	  parseargs
 *
 *	FUNCTION
 *    This function parses the arguments in argv
 *
 *  SYNOPSIS
 *    stderror = parseargs(int argc, char *argv[])
 *
 *  RESULT
 *    0 if successful else stderror
 *
 *  BUGS
 *
 *  TODO
 *
 *  SEE ALSO
 *    help()
 *    copyright()
 *
 *******
 */

  int count1,count2;
  int inno=-1;
  int outno=-1;
  int recognized;
  char *outname;

	/*  set defaults  */

  options.people=1; /* default=solo juggling */
	options.solo_mode = 0; // default: not set
  options.objektnr=0;
  options.handm=0;
  options.verz=0;
  options.beginwait=-1;
  options.infile=0;
 
  options.debug=FALSE; 
  options.hold=FALSE; 
  options.mark=FALSE; 
  options.nostart=FALSE; 
  options.sync=FALSE; 
  options.wrap=FALSE; 
 
  options.quiet=FALSE;
 
  options.radius=1.5;
  options.throwspec=0;
	options.multihandsiteswap=2;  // default: juggle with 2 hands

  if (argc<2){
    help();
    return 1;
  }
  if ((argc==2) & ((argv[1][1]=='c') || (argv[1][1]=='C')) && (argv[1][0]=='-')){
    copyright();
    return 1;
  }
  for(count1=1;count1<argc;count1++){
    if(argv[count1][0]=='-'){
      recognized=FALSE;
      switch(argv[count1][1]){			/* options with additional parameters */
      case 'e' : recognized=TRUE;
        if(sscanf(&argv[count1][2],"%d",&(options.beginwait))!=1){
          printf("Could not read waittime using 0\n");
          options.beginwait=0;
        }
        break;
       
      case 'h' : recognized=TRUE;
        options.handm=count1;			/* the checking is later */
        break;

      case 'H' : recognized=TRUE;  // WR
        options.throwspec = SS_HANDS;
        options.handm=count1;			/* the checking is later */
        break;
       
      case 'i' : recognized=TRUE;
        options.infile=count1;			/* read siteswap from file */
        inno=0;
        break;
       
      case 'm' : recognized=TRUE;  // WR
				{ 
					int na;
					na = sscanf(&argv[count1][2],"%d",&(options.multihandsiteswap));
					if ((na != 1) || (options.multihandsiteswap & 1)) {
						printf("Could not read multihandsiteswap (-m). Using 2 instead.\n");
						options.multihandsiteswap=2;
					} 
				}
        break;

			case 'o' : recognized=TRUE;
        if(sscanf(&argv[count1][2],"%d",&(options.objektnr))!=1){
          printf("Could not read objectnumber. Using balls\n");
          options.objektnr=6;
        }
        break;
       
      case  'p' :  recognized=TRUE;
        if(sscanf(&argv[count1][2],"%d",&(options.people))!=1){
          printf("I assume 2 people passing\n");
          options.people=2;
        }
				if (options.people == 1) 
					options.solo_mode = 1;  // this overrides options.people later
        break;
       
      case  'r' :  // WR
        recognized=TRUE;
        if(sscanf(&argv[count1][2],"%lf",&(options.radius))!=1) {
          printf("Could not read radius. Using 1.5m instead.\n");
          options.radius=1.5;
        }
        break;

      case 'S' : recognized=TRUE;  // WR
        options.throwspec = SS_SITESWAP;
        options.handm=count1;			/* the checking is later */
        break;

      case 't': // WR
        recognized=TRUE;
        break;
       
      default:   /* eine der unteren Optionen */
        break;
      }
      if (!recognized){
        for(count2=1;argv[count1][count2]!='\0';count2++){
  	      switch(argv[count1][count2]){
          case 'd' :
            options.debug=TRUE; 
            break;
          case 'f' :
            printf("Error: Wrong Error 734!\n");
            return 3;
          case 'l' :
            options.hold=TRUE; 		/* Longer handhold */
            break;
/*				# MEANWHILE; THIS COLLIDES WITH multihandsiteswap
					case 'm' :
            options.mark=TRUE; 		// mark paths with different coloured balls 
            break;
*/
					case 'n' :
            options.nostart=TRUE; 
            break;
          case 'q' :
            options.quiet=TRUE;
            break;
          case 'r' :
            break;
          case 's' :
            options.sync=TRUE; 
            break;
          case 'w' :
            options.wrap=TRUE; 
            break;
            
          default:    fprintf(stderr,"the option %c is not supported",argv[count1][count2]);
          }
        }
      }
    }
    else{
      if(inno==-1)inno=count1;
      else
        if(outno==-1)outno=count1;
    }
  }
  if(inno==-1){
    fprintf(stderr,"\nCould not recognize siteswap string\n"
            "better you read the help text\n");
    help();
    return 2;
  }
  if (outno==-1){
    outname=DEFOUTNAME;
  }
  else
    {
      outname=argv[outno];
    }
  options.outname=outname;
  options.sswap=inno;  // pointer to orig. siteswap pattern
 
	if (options.solo_mode)
		options.people = 1;
	
	// WRs modification for multi handed siteswaps: 21oct04
  if (options.multihandsiteswap == 2) {
		strncpy(options.inputsiteswap, argv[inno], SS_LEN); /* INPUT sichern */
	} else {  // more than 2 hands juggle this siteswap.
		strncpy(options.inputsiteswap,
			convertMHSS(argv[inno],options.multihandsiteswap/2), SS_LEN);
		options.people = options.multihandsiteswap/2;
	}
 
  if(options.debug){
    printf("site-swap : %s\n",argv[inno]);
		printf("input-ss  : %s\n",options.inputsiteswap);
    printf("Save to   : %s\n",outname);
  }
 
  return 0;
}

/*E*/
/*-----------------------------------------*/
/*A*/int self (int a, int b) {  // in jongl there is a completely different procedure with this name!

/* returns 1 if a and b are hands of the same juggler
*
* is different from jongl's view of numbering (L1 R1 L2 R2 L3 R3...)
* Here: L1 L2 L3 R1 R2 R3 !
*/

	int num_jug = options.multihandsiteswap/2;

	if (a%num_jug == b%num_jug) return 1;

	return 0;
}
/*E*/
/*-----------------------------------------*/

